<?php 
	
if ( !defined( 'ABSPATH' ) ) {
	exit(); // Exit if accessed directly
}

class RST_Reporting_Pro {
	
	public function __construct() {
		
    	$this->init();	    	
	}

	private function init() {
		
		$this->includes();
	}
	
	private function includes() {
		
		include_once 'includes/class-export-csv.php';
		include_once 'includes/class-settings-pro.php';
		include_once 'includes/class-products-manager-pro.php';
		
		// Reporting Settings class
		new RST_Reporting_Settings_Pro();
		
		// Products extention class
		new RST_Reporting_Products_Pro();
		
		// Includes files that extend files from free version/other modules
		add_filter( 'init',  array( $this, 'add_file_includes' ), 30, 1 );
		
		// Adds modules pro assets
		add_filter( 'it_rst_backend_assets',  array( $this, 'add_reporting_pro_rooster_backend_assets' ), 30, 1 );
		
		// Adds dashboard fullscreen feature
		add_filter( 'it_rst_reporting_dashboard_options_menu', array( $this, 'add_reporting_rooster_pro_fullscreen' ), 10, 1 );
		
		// Adds dashboard fullscreen feature
		add_filter( 'it_rst_reporting_dashboard_options_menu', array( $this, 'add_reporting_rooster_pro_reload' ), 10, 1 );
		
		// Adds search buttons to dashboard
		add_filter( 'it_rst_reporting_dashboard_menu_buttons', array( $this, 'add_reporting_rooster_pro_search_buttons' ), 10, 1 );
		
		// Adds export csv feature
		add_filter( 'it_rst_reporting_dashboard_table_menu', array( $this, 'add_reporting_rooster_pro_export_csv' ), 10, 1 );
		
		// Allows grid to be draggable
		add_filter( 'it_rst_reporting_dashboard_grid_classes', array( $this, 'add_reporting_rooster_pro_draggable' ), 10, 1 );
		
		// Adds module pro endpoints
		add_filter( 'it_rst_available_endpoints', 		   array( $this, 'add_reporting_rooster_pro_endpoints' ), 10, 1 );
		
		// Allows grids to be live updated
		add_filter( 'it_rst_reporting_dashboard_fastfacts_class', array( $this, 'add_reporting_rooster_pro_live_update' ), 10, 2 );
		add_filter( 'it_rst_reporting_dashboard_overview_class', array( $this, 'add_reporting_rooster_pro_live_update' ), 10, 2 );
		add_filter( 'it_rst_reporting_dashboard_table_class', array( $this, 'add_reporting_rooster_pro_live_update' ), 10, 2 );
				
		// Add CSV export feature
		add_action( 'it_rst_admin_menu',  array( $this, 'add_csv_export_page' ) );
		
		// FullScreen Shortcode
		add_shortcode( 'rst_fullscreen',  array( $this, 'rst_fullScreen' ) );
		
		// Dashboard Reload Data Shortcode
		add_shortcode( 'rst_reload_data', array( $this, 'rst_reloadData' ) );
		
		// Export CSV Shortcode
		add_shortcode( 'rst_export_csv',  array( $this, 'rst_exportCSV' ) );
	}
	
	public function add_file_includes() {
		
		include_once 'endpoints/class-reporting-box-pro.php';
		include_once 'endpoints/class-reporting-chart-pro.php';
		include_once 'endpoints/class-reporting-table-pro.php';
		include_once 'endpoints/class-reporting-grid.pro.php';
		include_once 'includes/class-queries-pro.php';
	}
	
	public function add_reporting_pro_rooster_backend_assets( $assets ) {
		
		global $it_rst_reporting_settings;
				
		$permissions = IT_RST_Permissions::get_instance();
    		    		    		    		    
	    // Show menu and submenus only to users with selected roles
	    if ( $permissions->check_permission( 'reporting' ) ) {
		    
		    $assets[] = array( 'slug' => 'rst_calendar_js', 'path' => 'modules/reporting-pro/assets/js/rst_calendar.js', 'type' => 'js', 'scope' => array( 'dashboard_on' ) );
		    
		    $assets[] = array( 'slug' => 'rst_utils_pro_js', 'path' => 'modules/reporting-pro/assets/js/rst_utils_pro.js', 'type' => 'js', 'scope' => array( 'dashboard_on' ) );
		    
		    $assets[] = array( 'slug' => 'rst_grid_pro_js', 'path' => 'modules/reporting-pro/assets/js/rst_grid_pro.js', 'type' => 'js', 'scope' => array( 'dashboard_on' ) );
		    
		    $assets[] = array( 'slug' => 'rst_feature_timers_js', 'path' => 'modules/reporting-pro/assets/js/rst_feature_timers.js', 'type' => 'js', 'scope' => array( 'plugin_pages' ) );
		    
		    $assets[] = array( 'slug' => 'rst_admin_pro_css', 'path' => 'modules/reporting-pro/assets/css/style.css', 'type' => 'css', 'scope' => array( 'dashboard_on' ) );	
		}
		
		return $assets;
	}
	
	public function add_reporting_rooster_pro_fullscreen( $html ) {
		
		if ( !wp_is_mobile() ) {
			$html .= '<div class="rst_view_fullscreen">' . do_shortcode( '[rst_fullscreen]' ) . ' <span>'. __( 'Fullscreen', 'ithemes-sales-accelerator' ) . '</span></div>';
		}
		
		return $html;
	}
	
	public function add_reporting_rooster_pro_reload( $html ){
		
		if ( !wp_is_mobile() ) {
			$html .= '<div class="rst_view_reload refresh_data">' . do_shortcode( '[rst_reload_data]' ) . ' <span>'. __( 'Refresh Data', 'ithemes-sales-accelerator' ) . '</span></div>';
		}
		
		return $html;
	}
	
	public function add_reporting_rooster_pro_search_buttons( $html ) {
		
		$pluginURL = plugin_dir_url( __FILE__ );
		$html .='<div class="search_buttons calendarBtt inline">	
						<div class="rst_menu_calendar">' . do_shortcode( '[rst_menu_calendar]' ) .'</div>
					</div>
					<div class="search_buttons inline">
						<div class="today">' . __( 'Today', 'ithemes-sales-accelerator' ) . '</div>
						<div class="yesterday">' . __( 'Yesterday', 'ithemes-sales-accelerator' ) . '</div>
						<div class="30days active">' . __( '30 Days', 'ithemes-sales-accelerator' ) . '</div>
					</div>';
					
		return $html;
	}
	
	public function add_reporting_rooster_pro_export_csv( $html ) {
		
		$html .= '<div class="rst_export_csv">' . do_shortcode( '[rst_export_csv]' ) . ' ' . __( 'Export to CSV', 'ithemes-sales-accelerator' ) . '</div>';
		return $html;
	}
	
	public function add_csv_export_page() {
			
		// Export CSV
        add_submenu_page( null, __( 'Sales Accelerator - Export CSV', 'ithemes-sales-accelerator' ), __( 'Export', 'ithemes-sales-accelerator' ), 'read', 'ithemes-sales-accelerator-plugin-export', array(
            $this,
            'export_csv'
        ) );
	}
	
	public function export_csv() {
		
	    $slug = isset( $_GET["slug"] ) ? $_GET["slug"]  : '';
	    if ( $_GET["dateStart"] == "empty" ) { 
	    	$dateStart = date('Y-m-d', ( strtotime ( '-30 days' ) ) );
	    }
	    else {
	    	$dateStart = $_GET["dateStart"];
	    }
	    if ( $_GET["dateEnd"] == "empty" ) {
	    	$dateEnd = date('Y-m-d');
	    }
	    else {
	    	$dateEnd = $_GET["dateEnd"];
	    }
	    	
		if( isset( $slug ) && isset( $dateStart ) && isset( $dateEnd ) ){
		    $controller = new RST_Reporting_CSV();
	        $controller->exportToCSV( $slug, $dateStart, $dateEnd );
        }
	}
	
	public function add_reporting_rooster_pro_draggable( $class ) {
		
		return '';
	}
	
	public function add_reporting_rooster_pro_live_update( $class, $item ) {
		
		global $it_rst_reporting_settings, $it_rst_reporting_feature_settings;
		
		// Check if Live Update option is active
		$liveUpdate = $it_rst_reporting_settings["live_update"];
		
		if ( $liveUpdate == "1" ) {
        	$value = $it_rst_reporting_feature_settings[ $item ]["live"];
        }
        else {
        	$value = '0';
        }
        	
        $class .= " data-gs-liveUpdate='$value'";
		return $class;
	}
	
	public function add_reporting_rooster_pro_endpoints( $endpoints ) {
		
		$base = 'reporting';
		$endpoints['box']['class']   	  = 'IT_RST_RP_Box_Pro';
		$endpoints['chart']['class'] 	  = 'IT_RST_RP_Chart_Pro';
		$endpoints['table']['class'] 	  = 'IT_RST_RP_Table_Pro';
		$endpoints['gridlayout']['class'] = 'IT_RST_RP_Grid_Pro';
		$endpoints['gridlayout']['http_endpoint'][] = array( 'method' => WP_REST_Server::READABLE, 'endpoint' => "$base/savegrid", 'callback' => 'savegrid_json_endpoint', 'module' => $base );
				
		return $endpoints;
	}
	
	public function rst_fullScreen() {
		
		return '
<svg width="17px" height="14px" viewBox="0 0 17 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" style="vertical-align:sub;">
    <!-- Generator: Sketch 46 (44423) - http://www.bohemiancoding.com/sketch -->
    <desc>Created with Sketch.</desc>
    <defs></defs>
    <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
        <g id="rst_view_fullscreen" fill-rule="nonzero" fill="#C2CCD4">
            <g id="Group">
                <polygon id="Shape" points="16.4705882 5.07425882 14.1880941 5.07425882 14.1880941 2.28249412 11.3963294 2.28249412 11.3963294 0 16.4705882 0"></polygon>
                <polygon id="Shape" points="16.4705882 13.8611529 11.3963294 13.8611529 11.3963294 11.5788235 14.1880941 11.5788235 14.1880941 8.78672941 16.4705882 8.78672941"></polygon>
                <polygon id="Shape" points="5.07442353 13.8609882 0 13.8609882 0 8.78672941 2.28249412 8.78672941 2.28249412 11.5786588 5.07442353 11.5786588"></polygon>
                <polygon id="Shape" points="2.28249412 5.07425882 0 5.07425882 0 0 5.07442353 0 5.07442353 2.28249412 2.28249412 2.28249412"></polygon>
            </g>
        </g>
    </g>
</svg>';
	}
	
	// Reload Data Icon
	public function rst_reloadData() {
			
			return '
<svg width="19px" height="16px" viewBox="0 0 19 16" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:sketch="http://www.bohemiancoding.com/sketch/ns">
    <!-- Generator: Sketch 3.3.3 (12072) - http://www.bohemiancoding.com/sketch -->
    <title>noun_584832_cc</title>
    <desc>Created with Sketch.</desc>
    <defs></defs>
    <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd" sketch:type="MSPage">
        <g id="rst_view_reload_data" sketch:type="MSLayerGroup" fill="#C2CCD4">
            <g id="Group" sketch:type="MSShapeGroup">
                <g>
                    <path d="M5.9670001,3.11610005 C6.98360012,2.34260004 8.19910014,1.92270003 9.48090016,1.92270003 C12.6854002,1.92270003 15.2932003,4.53050008 15.2932003,7.73500013 C15.2932003,7.86760014 15.2932003,8.00020014 15.2711003,8.11070014 L14.9838003,7.82340014 C14.6523003,7.46980013 14.0777002,7.46980013 13.7241002,7.80130013 C13.3705002,8.13280014 13.3705002,8.70740015 13.7020002,9.06100016 L16.0225003,11.4257002 L18.5861003,9.10520016 C18.9397003,8.77370015 18.9839003,8.22120014 18.6524003,7.84550014 C18.3209003,7.49190013 17.7684003,7.44770013 17.3927003,7.77920013 L17.0391003,8.11070014 L17.0391003,7.73500013 C17.0391003,3.55810006 13.6357002,0.154700003 9.45880016,0.154700003 C7.77920013,0.154700003 6.21010011,0.685100012 4.86200008,1.70170003 C4.46420008,1.98900003 4.39790008,2.54150004 4.68520008,2.93930005 C5.01670009,3.33710006 5.5913001,3.40340006 5.9670001,3.11610005 L5.9670001,3.11610005 Z" id="Shape"></path>
                    <path d="M0.994500017,7.84550014 C1.21550002,7.84550014 1.41440002,7.77920013 1.59120003,7.62450013 L1.92270003,7.31510013 C1.92270003,7.44770013 1.90060003,7.60240013 1.90060003,7.73500013 C1.90060003,11.9119002 5.30400009,15.3153003 9.48090016,15.3153003 C11.2047002,15.3153003 12.8180002,14.7628003 14.1661002,13.7020002 C14.5418003,13.3926002 14.6081003,12.8401002 14.3208002,12.4644002 C14.0114002,12.0887002 13.4589002,12.0224002 13.0832002,12.3097002 C12.0445002,13.1274002 10.8069002,13.5473002 9.50300016,13.5473002 C6.29850011,13.5473002 3.69070006,10.9395002 3.69070006,7.73500013 C3.69070006,7.58030013 3.69070006,7.42560013 3.71280006,7.27090013 L4.02220007,7.58030013 C4.35370008,7.93390014 4.92830009,7.93390014 5.28190009,7.60240013 C5.6355001,7.27090013 5.6355001,6.69630012 5.30400009,6.34270011 L2.96140005,3.97800007 L0.397800007,6.29850011 C0.0442000008,6.63000011 3.88578059e-16,7.18250012 0.331500006,7.55820013 C0.508300009,7.75710013 0.751400013,7.84550014 0.994500017,7.84550014 L0.994500017,7.84550014 Z" id="Shape"></path>
                </g>
            </g>
        </g>
    </g>
</svg>';
	}
		
	
	public function rst_exportCSV() {
		
		return '
<svg width="19px" height="14px" viewBox="0 0 19 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
    <!-- Generator: Sketch 46 (44423) - http://www.bohemiancoding.com/sketch -->
    <desc>Created with Sketch.</desc>
    <defs></defs>
    <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
        <g id="csv_image" fill-rule="nonzero" fill="#C2CCD4">
            <path d="M19,7.009 L14.6666667,12.015 L14.6666667,8 L6,8 L6,6 L14.6666667,6 L14.6666667,2 L19,7.009 Z M1.46666667,3 L1.46666667,11 C1.46666667,11.552 1.7952,12 2.2,12 L11,12 L11,14 L2.2,14 C0.984866667,14 0,12.657 0,11 L0,3 C0,1.343 0.984866667,0 2.2,0 L11,0 L11,2 L2.2,2 C1.7952,2 1.46666667,2.448 1.46666667,3 Z" id="Shape"></path>
        </g>
    </g>
</svg>';
	}		
}