<?php
	
if ( !defined( 'ABSPATH' ) ) {
	exit(); // Exit if accessed directly
}

class IT_RST_RP_Chart_Pro extends IT_RST_RP_Chart {
    
    // Singleton design pattern
    protected static $instance = NULL;
    
    // Method to return the singleton instance
    public static function get_instance() {
	    
        if ( null == self::$instance ) {
            self::$instance = new self();
        }
        
        return self::$instance;
    }

    public function __construct() {}
    
    // Orders Payed vs Pending
    public function chart_ordersPvP( $renderHtml, $dateStart, $dateEnd ) {

		$content = RST_Reporting_Queries_Pro::get_instance()->ordersPvP( $dateStart, $dateEnd );

		if ( $renderHtml == '1' ) {
			$value = ( $content['placed_per'] > $content['payed_per'] ? $content['placed_per'] : $content['payed_per'] );
			$html = '<div class="inline pt0"><h1>' . __( 'Orders Placed vs Orders Payed', 'ithemes-sales-accelerator' ) .'</h1><span class="selected_date">30 Days</span></div>
		            	<div id="chartFixedHeightContainer_Axis"><canvas id="orders_placed_payed"></canvas><span class="valueDonut">' . $value . '%</span></div>
		            	<div style="margin-left: -20px;margin-right: -20px;">
		            		<div class="legend1"><span class="legend_axis blue"></span> <span class="txt">' . __( 'Orders Placed', 'ithemes-sales-accelerator' ) .'</span> <span class="value">' . $content['placed_val'] . ' (' . $content['placed_per'] . '%)</span></div>
		            		<div class="legend2"><span class="legend_axis green">&nbsp;</span> <span class="txt">' . __( 'Orders Payed', 'ithemes-sales-accelerator' ) .'</span> <span class="value">' .  $content['payed_val'] . ' (' . $content['payed_per'] . '%)</span></div>
					</div>';
			return array( 'html' => $html, 'values' => array( 'label1' => 'Payed', 'value1' => $content['payed_per'], 'label2' => 'Placed', 'value2' => $content['placed_per'] ) );
		}
		else {
			$reporting_features = RST_Reporting_Features::get_instance();
			$type_chart 		= $reporting_features->is_lines_chart( 'orders_placed_payed' );
			$app_content = array();
			
			foreach ( $content as $k => $v ) {
				$app_content[] = array( 'name' => $k, 'total' => $v, 'total_app' => $v );
			}
			
			return array( 'columns' => array( 'placed_val' => 'Orders Placed', 'payed_val' => 'Orders Payed', 'payed_per' => 'Payed (%)', 'placed_per' => 'Placed (%)' ), 'type_chart' => $type_chart, 'values' => $app_content );
		}	
    }
    
    // Returning vs New
    public function chart_rn( $renderHtml, $dateStart, $dateEnd ) {

		$content = RST_Reporting_Queries_Pro::get_instance()->returningvsnew( $dateStart, $dateEnd );

		if ( $renderHtml == '1' ) {
			$value = ( $content['new_per'] > $content['customer_per'] ? $content['new_per'] : $content['customer_per'] );
			$html = '<div class="inline pt0"><h1>' . __( 'Returning vs New', 'ithemes-sales-accelerator' ) .'</h1><span class="selected_date">30 Days</span></div>
		            	<div id="chartFixedHeightContainer_Axis"><canvas id="returning_new"></canvas><span class="valueDonut">' . $value . '%</span></div>
		            	<div style="margin-left: -20px;margin-right: -20px;">
		            		<div class="legend1"><span class="legend_axis green"></span> <span class="txt">' . __( 'Returning Customers', 'ithemes-sales-accelerator' ) .'</span> <span class="value">' . $content['customer_val'] . ' (' . $content['customer_per'] . '%)</span></div>
		            		<div class="legend2"><span class="legend_axis blue">&nbsp;</span> <span class="txt">' . __( 'New Customers', 'ithemes-sales-accelerator' ) .'</span> <span class="value">' . $content['new_val'] . ' (' . $content['new_per'] . '%)</span></div>
					</div>';
			return array( 'html' => $html, 'values' => array( 'label1' => 'Returning', 'value1' => $content['customer_per'], 'label2' => 'New', 'value2' => $content["new_per"] ) );
		}
		else {			$reporting_features = RST_Reporting_Features::get_instance();
			$type_chart 		= $reporting_features->is_lines_chart( 'returning_new' );
			$app_content = array();
			
			foreach ( $content as $k => $v ) {
				$app_content[] = array( 'name' => $k, 'total' => $v, 'total_app' => $v );
			}
			
			return array( 'columns' => array( 'customer_val' => 'Returning Customers', 'new_val' => 'New Customers', 'customers_per' => 'Returning (%)', 'new_per' => 'New (%)' ), 'type_chart' => $type_chart, 'values' => $app_content );
		}
    }
    
    // Top Countries
    public function chart_topCountries( $renderHtml, $dateStart, $dateEnd ) {

		$content = RST_Reporting_Queries_Pro::get_instance()->topCountries( $dateStart, $dateEnd );

		if ( $renderHtml == '1' ) {
			$html = '<div class="inline pt0"><h1>' . __( 'Top Countries', 'ithemes-sales-accelerator' ) . '</h1><span class="selected_date">' . __( 'Last 30 Days', 'ithemes-sales-accelerator' ) . '</span></div>
		            	<div id="chartFixedHeightContainer_Axis"><canvas id="top_countries"></canvas></div>
					</div>';
			return array( 'html' => $html, 'values' => array( $content ) );
		}
		else {
			$reporting_features = RST_Reporting_Features::get_instance();
			$type_chart 		= $reporting_features->is_lines_chart( 'top_countries' );
			return array( 'columns' => array( 'name' => 'Country', 'total_app' => 'Total Sales' ), 'type_chart' => $type_chart, 'values' => $content );
		}		
    }
    
    // Spend by Day of Week
    public function chart_spendDay( $renderHtml, $dateStart, $dateEnd ) {

		$content = RST_Reporting_Queries_Pro::get_instance()->spendDay_Graph( $dateStart, $dateEnd );
		
		if ( $renderHtml == '1' ) {
			$html = '<div class="inline pt0"><h1>' . __( 'Spend by day of Week', 'ithemes-sales-accelerator' ) . '</h1><span class="selected_date">' . __( 'Last 30 Days', 'ithemes-sales-accelerator' ) . '</span></div>
		            	<div id="chartFixedHeightContainer_Axis"><canvas id="spend_day"></canvas></div>
					</div>';
			return array( 'html' => $html, 'values' => array( $content ) );
		}
		else {
			$reporting_features = RST_Reporting_Features::get_instance();
			$type_chart 		= $reporting_features->is_lines_chart( 'spend_day' );
			return array( 'columns' => array( 'name' => 'Spend by day of Week', 'total_app' => 'Total Sales' ), 'type_chart' => $type_chart, 'values' => $content );
		}
    }
    
    // Spend by Hour
    public function chart_spendHour( $renderHtml, $dateStart, $dateEnd ) {

		$content = RST_Reporting_Queries_Pro::get_instance()->spendHour_Graph( $dateStart, $dateEnd );
		
		if ( $renderHtml == '1' ) {
			$html = '<div class="inline pt0"><h1>' . __( 'Spend by Hour', 'ithemes-sales-accelerator' ) . '</h1><span class="selected_date">' . __( 'Last 30 Days', 'ithemes-sales-accelerator' ) . '</span></div>
		            	<div id="chartFixedHeightContainer_Axis"><canvas id="spend_hour"></canvas></div>
					</div>';
			return array( 'html' => $html, 'values' => array( $content ) );
		}
		else {
			$reporting_features = RST_Reporting_Features::get_instance();
			$type_chart 		= $reporting_features->is_lines_chart( 'spend_hour' );
			return array( 'columns' => array( 'name' => 'Spend by Hour', 'total_app' => 'Total Sales' ), 'type_chart' => $type_chart, 'values' => $content );
		}
    }
    
    // Spend Week
    public function chart_spendWeek( $renderHtml, $dateStart, $dateEnd ) {

		$content = RST_Reporting_Queries_Pro::get_instance()->spendWeek_Graph( $dateStart, $dateEnd );
		
		if ( $renderHtml == '1' ) {
			$html = '<div class="inline pt0"><h1>' . __( 'Spend by Week', 'ithemes-sales-accelerator' ) . '</h1><span class="selected_date">' . __( 'Last 30 Days', 'ithemes-sales-accelerator' ) . '</span></div>
		            	<div id="chartFixedHeightContainer_Axis"><canvas id="spend_week"></canvas></div>
					</div>';
			return array( 'html' => $html, 'values' => array( $content ) );
		}
		else {
			$reporting_features = RST_Reporting_Features::get_instance();
			$type_chart 		= $reporting_features->is_lines_chart( 'spend_week' );
			return array( 'columns' => array( 'name' => 'Spend by Week', 'total_app' => 'Total Sales' ), 'type_chart' => $type_chart, 'values' => $content );
		}
    }
   
    // Send result by JSON Endpoint
    public function contentChart_json_endpoint( $request ) {
	    
	    // Get params
	    $params = $request->get_params();
	    $renderHtml = isset( $params['renderHtml'] ) ? $params['renderHtml'] : '';
	    $chartName 	= isset( $params['slug'] ) 		 ? $params['slug'] 		 : '';
	    $dateStart 	= isset( $params['dateStart'] )  ? $params['dateStart']  : date('Y-m-d',(strtotime ( '-30 days' ) ) );
	    $dateEnd 	= isset( $params['dateEnd'] ) 	 ? $params['dateEnd'] 	 : date('Y-m-d');

	    // Recent Orders
	    switch ( $chartName ) {
			
		    // Orders Payed vs Pending
		    case 'orders_placed_payed':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_ordersPvP( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_ordersPvP( '', $dateStart, $dateEnd );
				}
			break;
			
			// Customers vs Guests
		    case 'customers_guests':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_cg( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_cg( '', $dateStart, $dateEnd );
				}
			break;
			
			// Returning vs New		    
			case 'returning_new':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_rn( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_rn( '', $dateStart, $dateEnd );
				}
			break;
			
			// Top Countries		    
			case 'top_countries':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_topCountries( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_topCountries( '', $dateStart, $dateEnd );
				}
			break;
			
			// Shipping Method		    
			case 'shipping_method':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_shippingMethod( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_shippingMethod( '', $dateStart, $dateEnd );
				}
			break;
			
			// Spend Day Week		    
			case 'spend_day':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_spendDay( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_spendDay( '', $dateStart, $dateEnd );
				}
			break;
			
			// Spend by Week		    
			case 'spend_week':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_spendWeek( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_spendWeek( '', $dateStart, $dateEnd );
				}
			break;
			
			// Spend by Hour		    
			case 'spend_hour':
		    	if ( $renderHtml == '1' ) {
		    		$data = $this->chart_spendHour( $renderHtml, $dateStart, $dateEnd );
		    	}
		    	else {
					$data = $this->chart_spendHour( '', $dateStart, $dateEnd );
				}
			break;
				
			default :
		    	return new IT_REST_Response( array( 'result' => false, 'data' => array() )  , 200 );
			break;
		}
			    
	    return new IT_REST_Response( array( 'result' => true, 'data' => $data )  , 200 );
    }
}