<?php
	

if ( !defined( 'ABSPATH' ) ) {
	exit(); // Exit if accessed directly
}

/**
* This class handles reporting csv exports
**/

class RST_Reporting_CSV {
	    	
	public function __construct() {}
	
	public function exportToCSV( $slug, $dateStart, $dateEnd ) {
			
		$filename = $slug . '_' . $dateStart . '_' . $dateEnd . '.csv';
		ob_clean();
		header('Content-type: application/csv');
		header('Content-Disposition: attachment; filename='.$filename);
							
		global $wpdb;
			
		switch ( $slug ) {
			case 'zone_distribution':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->zoneDist_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Zone', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'state', 'total', 'num_orders', 'num_products', 'coupon' );
		        break;
			case 'category_by_country':
				$content = RST_Reporting_Queries_Pro::get_instance()->categoryCountry_Table( $dateStart, $dateEnd , '1' );
				$headers = array( 'Country', 'Category', 'Total', 'Num. Products', 'Total Coupons' );
				$content_keys = array( 'country', 'category', 'total', 'num_products', 'coupon' );
				break;
		    case 'best_customers':
		        $content = RST_Reporting_Queries_Pro::get_instance()->bestCustomers( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Number', 'Name', 'Email', 'Orders', 'Spend', 'Country' );
		        $content_keys = array( 'customer_id', 'name', 'email', 'orders', 'spend', 'country' );
		        break;
		    case 'available_quantities_ff':
		        $content = RST_Reporting_Queries_Pro::get_instance()->availableQtt_Table( '1' );
		        $headers = array( 'Number', 'Image', 'Name', 'Stock', 'Price', 'Total Value' );
		        $content_keys = array( 'product_id', 'image', 'name', 'stock', 'price', 'total_value' );
		        break;
		    case 'best_category':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->bestCategories_Table( $dateStart, $dateEnd , '1' );
		        $headers = array(  'Product Type', 'Category', 'Total', 'Total Quantities', 'Total Coupons' );
		        $content_keys = array( 'product_type', 'category', 'total', 'total_qtt', 'avg_price' );
		        break;
		    case 'total_coupons':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->coupons_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Coupon', 'Total', 'Num. Orders', 'Num. Products', 'AVG Price' );
		        $content_keys = array( 'coupon_name', 'total', 'num_orders', 'num_products', 'coupons' );
		        break;
		    case 'best_product':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->bestProducts_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Number', 'Image', 'Name', 'Total', 'Category', 'Num. Products' );
		        $content_keys = array( 'product_id', 'image', 'product', 'total', 'category', 'num_products' );
		        break;
		    case 'payment_methods':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->paymentMethod_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Payment', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'payment', 'total', 'num_orders', 'num_products', 'coupons' );
		        break;
		    case 'out_of_stock':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->outofstock_Table( '1' );
		        $headers = array( 'Number', 'Image', 'Name', 'SKU', 'Price', 'Backorders' );
		        $content_keys = array( 'product_id', 'image', 'name', 'sku', 'price', 'backorders' );
		        break;		        
		    case 'orders_operating_system':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->ordersOS_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Platform', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'platform', 'total', 'num_orders', 'num_products', 'coupons' );
		        break;	
		    case 'orders_browsers':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->ordersBrowsers_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Browser', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'browser', 'total', 'num_orders', 'num_products', 'coupons' );
		        break;	
		    case 'type_products':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->productType_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Type', 'Tota Salesl', 'Total Qtt', 'Total Qtt Refunded', 'Avg Price' );
		        $content_keys = array( 'product_type', 'total', 'total_qtt', 'total_ref', 'avg_price' );
		        break;	
		    case 'top_countries':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->topCountries_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Country', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'country', 'total', 'num_orders', 'num_products', 'coupons' );
		        break;
		    case 'spend_day':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->spendDay_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Day', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'day', 'total', 'num_orders', 'num_products', 'coupon' );
		        break;
		    case 'spend_week':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->spendWeek_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Week', 'Period', 'Total', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'week', 'period', 'total', 'num_products', 'coupon' );
		        break;
		    case 'spend_hour':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->spendHour_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Hour', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'hour', 'total', 'num_orders', 'num_products', 'coupon' );
		        break;
		    case 'total_refunds':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->totalRefunds_Table( $dateStart, $dateEnd , '1' );
		        $headers = array( 'Order', 'Date', 'Status', 'Value Refunded', 'Total Order' );
		        $content_keys = array( 'order_id', 'datetime', 'status', 'value_refunded', 'total' );
		        break;
		    case 'orders_status':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->ordersStatus_Table( $dateStart, $dateEnd );
		        $headers = array( 'Status', 'Total', 'Num. Orders', 'Num. Products', 'Total Coupons' );
		        $content_keys = array( 'status', 'total', 'num_orders', 'num_products', 'coupon' );
		        break;
		    case 'in_stock':
		    	$content = RST_Reporting_Queries_Pro::get_instance()->inStock_Table( '1' );
		        $headers = array( 'Number', 'Name', 'Stock', 'Price', 'Total Value', 'Image' );
		        $content_keys = array( 'product_id', 'name', 'stock', 'price', 'total_value', 'image' );
		        break;
		}
										
        $f = tmpfile();
	
        fputcsv( $f, $headers, ',' );
		
		foreach ( $content as $item ) {
			$line = array();
			
			foreach( $content_keys as $v ){
				$line[] = isset( $item[$v] ) ? html_entity_decode( $item[$v], ENT_QUOTES, 'UTF-8' ) : '';
			}
			
			fputcsv( $f, $line, ',' );
		}
		
		/**
	     * rewrind the 'file' with the csv lines *
	     */
	    fseek( $f, 0 );

	    fpassthru( $f );
	    die();
		
	}

}