<?php
	
if ( !defined( 'ABSPATH' ) ) {
	exit(); // Exit if accessed directly
}

/**
* This class handles product creation/updates
**/

class RST_Reporting_Products_Pro {
	
	public function __construct() {
		
		add_action( 'init', array( $this, 'add_product_cost_field' ) );
	}
	
	public function add_product_cost_field() {
		
		global $it_rst_reporting_settings;
		if ( isset( $it_rst_reporting_settings['product_cost'] ) &&  $it_rst_reporting_settings['product_cost'] ) {
	        
			// Adds product cost field to simple products
			add_action( 'woocommerce_product_options_pricing', 	 array( $this,'add_product_cost_simple_product' ) );
			
			// Adds product cost field to variable products
			add_action( 'woocommerce_variation_options_pricing', array( $this, 'add_product_cost_variable_product'), 10, 3 );
			
			// Save product cost field
			add_action( 'woocommerce_process_product_meta', 	 array( $this,'add_product_cost_simple_product_save' ) );
			add_action( 'woocommerce_save_product_variation', 	 array($this, 'add_product_cost_variable_product_save'), 10, 2 );
		}
	}
	
	public function add_product_cost_simple_product_save( $post_id ) {
		
		if ( isset( $_POST['_it_rooster_product_cost'] ) ) {
			update_post_meta( $post_id, '_it_rooster_product_cost', esc_attr( $_POST['_it_rooster_product_cost'] ) );
		}
	}
	
	public function add_product_cost_variable_product_save( $post_id, $loop ) {
		
		if ( isset( $_POST['_it_rooster_product_cost'][ $post_id ] ) ) {
			$sanitized_price = sanitize_text_field( $_POST['_it_rooster_product_cost'][ $post_id ] );
			update_post_meta( $post_id, '_it_rooster_product_cost', $sanitized_price );
		}
	}
	
	public function add_product_cost_variable_product( $loop, $variation_data, $variation ) {
		
		$product_cost = get_post_meta( $variation->ID, '_it_rooster_product_cost', TRUE );
		echo '<div class="options_group">';
			
		woocommerce_wp_text_input(
			array(
				'id'          => "_it_rooster_product_cost[$variation->ID]",
				'class'       => 'show_if_variable wc_input_price',
				'label'       => __( 'Product Cost', 'ithemes-sales-accelerator' ),
				'type'        => 'text', 
				'desc_tip'    => 'true',
				'description' => __( 'Enter product cost (if any).', 'ithemes-sales-accelerator' ),
				'value'       => $product_cost,
			)
		);
						
		echo '</div>';
	}
	
	public function add_product_cost_simple_product() {
		
		global $post;
		$product = new IT_WC_Product( $post->ID );
		
		if ( !$product->is_type( 'variable' ) ) {
			echo '<div class="options_group">';
							
			woocommerce_wp_text_input(
				array(
					'id'          => '_it_rooster_product_cost',
					'class'       => 'show_if_simple short wc_input_price',
					'label'       => __( 'Product Cost', 'ithemes-sales-accelerator' ),
					'type'        => 'text', 
					'desc_tip'    => 'true',
					'description' => __( 'Enter product cost (if any).', 'ithemes-sales-accelerator' ),
				)
			);
				
			echo '</div>';
		}
	}	
}