<?php

if ( !defined( 'ABSPATH' ) ) {
	exit(); // Exit if accessed directly
}

class RST_Reporting_Queries_Pro extends RST_Reporting_Queries {
	
	// Singleton design pattern
    protected static $instance = NULL;
    
    // Method to return the singleton instance
    public static function get_instance() {
	    
        if ( null == self::$instance ) {
            self::$instance = new self();
        }
        
        return self::$instance;
    }
	
	public function __construct() {}
		   									
	/*
	 * Low Stock Products
	*/
	public function lowStock() {
		
		global $wpdb;
		$lowStock = get_option( 'woocommerce_notify_low_stock_amount' );
		$low = $wpdb->get_var( "select count(*) from " . $wpdb->postmeta . " where post_id in (select p.ID from " . $wpdb->posts . " p inner join " . $wpdb->postmeta . " pm on p.ID = pm.post_id where meta_key = '_manage_stock' and meta_value = 'yes' and post_status = 'publish') and meta_key = '_stock' and meta_value <= $lowStock" );	
						
		return floatval( $low );
	}
			
	/*
	 * Waiting for Payment
	*/
	public function waitingPayment() {
		
		global $wpdb;
		$totalType = $this->totalType();
		$waiting = $wpdb->get_var( "select sum($totalType) as total from " . IT_RST_REPORTING_ORDERS_DATABASE . " where status = 'pending'" );
		
		return  strip_tags( wc_price( $waiting ) );
	}
	
	/*
	 * Waiting to be Shipped
	*/
	public function waitingShipped() {
		
		global $wpdb;
		$totalType = $this->totalType();
		$shipped = $wpdb->get_var( "select sum($totalType) as total from " . IT_RST_REPORTING_ORDERS_DATABASE . " where status = 'processing'" );
		
		return  strip_tags( wc_price( $shipped ) );
	}
	
	/*
	 * In Stock
	*/
	public function inStock() {
		
		global $wpdb;
		$in = $wpdb->get_var( "select count(*) from " . $wpdb->postmeta . " where post_id in (select p.ID from " . $wpdb->posts . " p inner join " . $wpdb->postmeta . " pm on p.ID = pm.post_id where meta_key = '_manage_stock' and meta_value = 'yes' and post_status = 'publish') and meta_key = '_stock_status' and meta_value = 'instock'" );
		
		return $in;					
	}
	
	/*
	 * Average Markup
	*/
	public function averageMarkup() {
		
		global $wpdb;
		$avg = $wpdb->get_var( "select sum(markup)/count(*) as avg_markup from ( select post_id, price, cost, (price-cost)/cost*100 as markup from ( select post_id, meta_value as price, (select meta_value as price from " . $wpdb->postmeta . " where post_id =x.post_id and meta_key = '_it_rooster_product_cost') as cost from " . $wpdb->postmeta . " x where post_id in (select p.ID from " . $wpdb->posts . " p inner join " . $wpdb->postmeta . " pm on p.ID = pm.post_id where meta_key = '_it_rooster_product_cost' and meta_value <> 0 and post_status = 'publish') and meta_key = '_regular_price' ) as y ) as z" );
		
		if ( !$avg || !is_numeric( $avg ) ){
			$avg = 'N/A';
			return $avg;
		}
		
		return number_format( (float) $avg, 2, '.', '' ) . '%';
	}
							
	/*
	 * New Clients
	*/
	public function newClients( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$days = $this->diffDays( $startDate,$endDate );
		
		// What query to execute
		if ( $days == 0 ) {
			$select = "select count(customer) as customers, date, hour(date) as hour from " . IT_RST_REPORTING_ORDERS_DATABASE . " ro inner join " . $wpdb->users . " u on ro.customer = u.id WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and DATE_FORMAT(date,'%%Y-%%m-%%d') <= DATE_FORMAT(user_registered,'%%Y-%%m-%%d') group by hour(date) order by hour(date) asc";
		}
		else{
			$select = "select count(customer) as customers, date from " . IT_RST_REPORTING_ORDERS_DATABASE . " ro inner join " . $wpdb->users . " u on ro.customer = u.id WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and DATE_FORMAT(date,'%%Y-%%m-%%d') <= DATE_FORMAT(user_registered,'%%Y-%%m-%%d') group by day(date), month(date), year(date) order by date asc";
		}

		$query 	  = $wpdb->prepare( $select, $startDate, $endDate );	
		$newUsers = $wpdb->get_results( $query );
		
		$newUsersA = array();
		$total 	   = 0;
		$variation = array();
		$i 		   = 0;

		foreach ( $newUsers as $user ) {
			if ( $days == 0 ) {
				array_push( $newUsersA, array( 'value' => floatval($user->customers ), 'date' => $user->date, 'hour' => $user->hour ) );
			}
			else {
				array_push( $newUsersA, array( 'value' => floatval($user->customers ), 'date' => $user->date ) );
			}

			$total = $total + $user->customers;
			
			if ( $i != 0 ) {
				array_push( $variation, ( $user->customers/$newUsers[$i-1]->customers )-1 );
			}

			$i++;
		}
		
		$newData = $this->orderArray( $startDate, $endDate, $newUsersA );

		if ( $variation )
			$variationAvg = ( array_sum( $variation ) / count( $variation ) ) * 100;
		else
			$variationAvg = 0;
			
		array_multisort( $newData['dates'], SORT_ASC, $newData['allValues'] );
		return array( 'variation' => round( $variationAvg, 2 ), 'total' => $total, 'lines' =>  $newData['allValues'] );
		
	}						
	
	/*
	 * Number of Products Bought
	*/
	public function productsBought( $startDate, $endDate ){
		
		global $wpdb;	
		
		if ( !( $this->validateDate($startDate) && $this->validateDate( $endDate ) ) ) 
			return array();
			
		$days = $this->diffDays( $startDate,$endDate );
		$orderStatus = $this->orderStatus();
		
		// What query to execute
		if ( $days == 0 ) {
			$select = "select sum(quantity-quantity_refunded) as num_products, datetime, hour(datetime) as hour from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ", 'refunded' ) group by hour(datetime) order by datetime asc";
		}
		else {
			$select = "select sum(quantity-quantity_refunded) as num_products, DATE_FORMAT(datetime,'%%Y-%%m-%%d') as date from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ", 'refunded' ) group by day(date), month(date), year(date) order by date asc";
		}

		$query 		   = $wpdb->prepare( $select, $startDate, $endDate );	
		$totalProducts = $wpdb->get_results( $query );
	
		$totalProductsA = array();
		$total 			= 0;
		$variation 		= array();
		$i 				= 0;

		foreach ( $totalProducts as $product ) {
			if ( $days == 0 ) {
				array_push( $totalProductsA, array( "value" => floatval( $product->num_products ), "date" => $product->date, "hour" => $product->hour ) );
			}
			else {
				array_push( $totalProductsA, array( "value" => floatval( $product->num_products ), "date" => $product->date ) );
			}

			$total = $total + $product->num_products;
			
			if ( $i != 0 ) {
				array_push( $variation, ( $product->num_products/$totalProducts[$i-1]->num_products ) - 1 );
			}

			$i++;
		}
		
		$newData = $this->orderArray( $startDate, $endDate, $totalProductsA );

		if ( $variation ) {
			$variationAvg = ( array_sum( $variation ) / count( $variation ) ) * 100;
		}
		else {
			$variationAvg = 0;
		}
			
		array_multisort( $newData['dates'], SORT_ASC, $newData['allValues'] );
		return array( 'variation' => round( $variationAvg, 2 ), 'total' => $total, 'lines' =>  $newData['allValues'] );
	}
	
	/*
	 * Available Quantities - Table
	*/
	public function availableQtt_Table( $records = '0' ) {
	
		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$query = "select post_id as ID from " . $wpdb->postmeta . " where post_id in (select p.ID from " . $wpdb->posts . " p inner join " . $wpdb->postmeta . " pm on p.ID = pm.post_id where meta_key = '_manage_stock' and meta_value = 'yes' and post_status = 'publish') and meta_key = '_stock' and meta_value > 0";
		
		$products = $wpdb->get_results( $query );
		$productsA = array();
		
		foreach ( $products as $product ) {
			$_product = wc_get_product( $product->ID );
			if ( $_product->is_type( 'variable' ) == true ) {
				$variations = $_product->get_available_variations();
				foreach ( $variations as $key => $value ) {
					$_product 	 = wc_get_product( $value['variation_id'] );
					$name 	     = get_the_title(  $value['variation_id'] );
					$stock 		 = ( $_product->get_stock_quantity() ) ? $_product->get_stock_quantity() : 0;
					$price 		 = $_product->get_price();
					$total_value = ( $stock > 0 ) ? $price * $stock : $price;
					$image 		 = wp_get_attachment_image_src( get_post_thumbnail_id( $value['variation_id'] ), 'single-post-thumbnail' );
					array_push( $productsA, array( 'product_id' => $value['variation_id'], 'name' => $name, 'stock' => $stock, 'price' =>  strip_tags( wc_price( $price ) ), 'total_value' =>  strip_tags( wc_price( $total_value ) ), 'image' => $image[0] ) );
				}
			}
			else {
				$_product 	 = wc_get_product( $product->ID );
				$name 	     = get_the_title(  $product->ID );
				$stock 		 = ( $_product->get_stock_quantity() ) ? $_product->get_stock_quantity() : 0;
				$price 		 = $_product->get_price();
				$total_value = ( $stock > 0 ) ? $price * $stock : $price;
				$image 	     = wp_get_attachment_image_src( get_post_thumbnail_id( $product->ID ), 'single-post-thumbnail' );
				array_push( $productsA, array( 'product_id' => $product->ID, 'name' => $name, 'stock' => $stock, 'price' =>  strip_tags( wc_price( $price ) ), 'total_value' =>  strip_tags( wc_price( $total_value ) ), 'image' => $image[0] ) );
		    }
		}
		return $productsA;
	}
	
	/*
	 * In Stock - Table
	*/
	public function inStock_Table( $records = '0' ) {
	
		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$query = "select post_id as ID from " . $wpdb->postmeta . " where post_id in (select p.ID from " . $wpdb->posts . " p inner join " . $wpdb->postmeta . " pm on p.ID = pm.post_id where meta_key = '_manage_stock' and meta_value = 'yes' and post_status = 'publish') and meta_key = '_stock_status' and meta_value = 'instock'";
		
		$products = $wpdb->get_results( $query );
		$productsA = array();
		
		foreach ( $products as $product ) {
			$_product = wc_get_product( $product->ID );
			if ( $_product->is_type( 'variable' ) == true ) {}				
			else {
				$_product 	 = wc_get_product( $product->ID );
				$name 	     = get_the_title(  $product->ID );
				$stock 		 = ( $_product->get_stock_quantity() ) ? $_product->get_stock_quantity() : 0;
				$price 		 = $_product->get_price();
				$total_value = ( $stock > 0 ) ? $price * $stock : $price;
				$image 	     = wp_get_attachment_image_src( get_post_thumbnail_id( $product->ID ), 'single-post-thumbnail' );
				array_push( $productsA, array( 'product_id' => $product->ID, 'name' => $name, 'stock' => $stock, 'price' =>  strip_tags( wc_price( $price ) ), 'total_value' =>  strip_tags( wc_price( $total_value ) ), 'image' => $image[0] ) );
			}
		}
		
		return $productsA;
	}
	
	/*
	 * Out of Stock - Table
	*/
	public function outofstock_Table( $records = '0' ) {
		
		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$query = "select post_id as ID from " . $wpdb->postmeta . " where post_id in (select p.ID from " . $wpdb->posts . " p inner join " . $wpdb->postmeta . " pm on p.ID = pm.post_id where meta_key = '_manage_stock' and meta_value = 'yes' and post_status = 'publish') and meta_key = '_stock_status' and meta_value = 'outofstock'";	
		
		$products = $wpdb->get_results( $query );
		$productsA = array();
		
		foreach ( $products as $product ) {
			$_product = wc_get_product( $product->ID );
			if ( $_product->is_type( 'variable' ) == true ) {
				$variations = $_product->get_available_variations();
				foreach ( $variations as $key => $value ) {
					$_product    = wc_get_product( $value['variation_id'] );
					$name 	     = get_the_title( $value['variation_id'] );
					$image 	     = wp_get_attachment_image_src( get_post_thumbnail_id( $value['variation_id'] ), 'single-post-thumbnail' );
					$sku 	     = get_post_meta( $value['variation_id'], '_sku', true );
					$backorders  = get_post_meta( $value['variation_id'], '_backorders', true );
					$price 		 = $_product->get_price();
					$total_value = ( $stock > 0 ) ? $price * $stock : $price;
					array_push( $productsA, array( "sku" => $sku, "product_id" => $value['variation_id'], "name" => $name, "stock" => $stock, "price" =>  strip_tags( wc_price( $price ) ), "total_value" =>  strip_tags( wc_price( $total_value ) ), "image" => $image[0] ) );
				}
			}
			else {
				$_product 	 = wc_get_product( $product->ID );
				$name 	  	 = get_the_title( $product->ID );
				$sku 	  	 = get_post_meta( $product->ID, '_sku', true );
				$image 	  	 = wp_get_attachment_image_src( get_post_thumbnail_id( $product->ID ), 'single-post-thumbnail' );
				$stock 	  	 = ( $_product->get_stock_quantity() ) ? $_product->get_stock_quantity() : 0;
				$price 	     = $_product->get_price();
				$backorders  = get_post_meta( $product->ID, '_backorders', true );
				$total_value = ( $stock > 0 ) ? $price * $stock : $price;
				array_push( $productsA, array( 'sku' => $sku, 'product_id' => $product->ID, 'name' => $name, 'backorders' => $backorders, 'price' =>  strip_tags( wc_price( $price ) ), 'image' => $image[0] ) );			
			}
		}
		return $productsA;
	}
	
	/*
	 * Orders by Browsers - Table
	*/
	public function ordersBrowsers_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) 
			return array();

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$orderStatus = $this->orderStatus();
		$totalType   = $this->totalType();

		$query = $wpdb->prepare( "select browser, sum(total) as total, count(order_id) as orders, sum(products) as products, sum(coupon) as coupon from (select order_id, browser, total_gross as total, order_id as orders, num_products as products, value_discount as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in (" . $orderStatus  . ", 'refunded') union all select e.order_id, o.browser, -sum(meta2), 0, 0, 0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in (" . $orderStatus  . ", 'refunded') group by order_id) x group by browser $records", $startDate, $endDate, $startDate, $endDate );	
		
		$browsers = $wpdb->get_results( $query );
		$browsersA = array();
		
		foreach ( $browsers as $browser ) {				
			$browser_name = ( $browser->browser ) ? $browser->browser : 'Other';
			array_push( $browsersA, array( 'browser' => $browser_name, 'total' =>  strip_tags( wc_price( $browser->total ) ), 'num_orders' => $browser->orders, 'num_products' => $browser->products, 'coupons' =>  strip_tags( wc_price( $browser->coupon ) ) ) );
		}
		
		return $browsersA;
	}
	
	/*
	 * Orders by OS - Table
	*/
	public function ordersOS_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select platform, sum(total) as total, count(order_id) as orders, sum(products) as products, sum(coupon) as coupon from (select order_id, platform, total_gross as total, order_id as orders, num_products as products, value_discount as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in (" . $orderStatus  . ", 'refunded')  union all select e.order_id, o.platform, -sum(meta2), 0, 0, 0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s' and order_status in (" . $orderStatus  . ", 'refunded') ) group by order_id) x group by platform $records", $startDate, $endDate, $startDate, $endDate );	
		
		$plataforms = $wpdb->get_results( $query );
		$plataformsA = array();
		
		foreach ( $plataforms as $plataform ) {				
			$platform_name = ( $plataform->platform ) ? $plataform->platform : 'Unknown';
			array_push( $plataformsA, array( 'platform' => $platform_name, 'total' =>  strip_tags( wc_price( $plataform->total ) ), 'num_orders' => $plataform->orders, 'num_products' => $plataform->products, 'coupons' =>  strip_tags( wc_price( $plataform->coupon ) ) ) );
		}
		
		return $plataformsA;
	}
	
	/*
	 * Coupons - Table
	*/
	public function coupons_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select name, sum(total) as total, count(order_id) as orders, sum(products) as products, sum(coupon) as coupon from (select order_id, coupon as name, total_gross as total, order_id as orders, num_products as products, value_discount as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in (" . $orderStatus  . ") union all select e.order_id, o.coupon, -sum(meta2), 0, 0, 0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in (" . $orderStatus  . ") group by order_id) x group by name $records", $startDate, $endDate, $startDate, $endDate );
		
		$coupons = $wpdb->get_results( $query );
		$couponsA = array();
		
		foreach ( $coupons as $coupon ) {
			
			// Add discounts that are associated with a coupon, remove otherwise
			if( $coupon->name ) {			
				array_push( $couponsA, array( 'coupon_name' => $coupon->name, 'total' =>  strip_tags( wc_price( $coupon->total ) ), 'num_orders' => $coupon->orders, 'num_products' => $coupon->products, 'coupons' =>  strip_tags( wc_price( $coupon->coupon ) ) ) );
			}
		}
		
		return $couponsA;
	}
	
	/*
	 * Payment Method - Table
	*/
	public function paymentMethod_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select payment, sum(total) as total, count(order_id) as orders, sum(products) as products, sum(coupon) as coupon from (select order_id, payment, total_gross as total, order_id as orders, num_products as products, value_discount as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in (" . $orderStatus  . ")  union all select e.order_id, o.payment, -sum(meta2), 0, 0, 0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in (" . $orderStatus  . ") group by order_id) x group by payment $records", $startDate, $endDate, $startDate, $endDate );	
		
		$payments = $wpdb->get_results( $query );
		$paymentsA = array();
		
		foreach ( $payments as $payment ) {				
			$payment_name = ( $payment->payment ) ? $payment->payment : 'N/A';
			
			array_push( $paymentsA, array( 'payment' => $payment_name, 'total' =>  strip_tags( wc_price( $payment->total ) ), 'num_orders' => $payment->orders, 'num_products' => $payment->products, 'coupons' =>  strip_tags( wc_price( $payment->coupon ) ) ) );
		}
		
		return $paymentsA;
	}
	
	/*
	 * Best Selling Products - Table
	*/
	public function bestProducts_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select x.product, (select name from " . IT_RST_REPORTING_PRODUCTS_DATABASE . " where product_id = x.product) as name, sum(total) as total, (select sum(quantity) from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " where product = x.product and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s')) as products, (select category from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " where product = x.product limit 1) as category from (select product, sum(price) as total from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_id in (select order_id from " . IT_RST_REPORTING_ORDERS_DATABASE . " where status in (" . $orderStatus  . ", 'refunded' ) and (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s')) group by product union all select REPLACE(meta1, '.000000', ''), -sum(meta2) from " . IT_RST_REPORTING_EVENTS_DATABASE . " where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in (" . $orderStatus  . ", 'refunded' ) and meta1 <> 0 group by meta1) x group by product $records", $startDate, $endDate, $startDate, $endDate, $startDate, $endDate, $startDate, $endDate );
					
		$products  = $wpdb->get_results( $query );
		$productsA = array();
		
		foreach ( $products as $product ) {		
			
			$category = ( $product->category ) ? $product->category : 'Uncategorized';
			$image = wp_get_attachment_image_src( get_post_thumbnail_id( $product->product ), 'single-post-thumbnail' );		
			array_push( $productsA, array( 'product' => $product->name, 'total' =>  strip_tags( wc_price( $product->total ) ), 'category' => $category, 'num_products' => $product->products, 'product_id' => $product->product, 'image' => $image[0] ) );
		}
		
		return $productsA;
	}
	
	/*
	 * Product Type - Table
	*/
	public function productType_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select product_type, sum(price-total_refunded) as total, sum(quantity) as qtt, sum(quantity_refunded) as qtt_ref, (sum(price)/sum(quantity)) as avgprice from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_id in (select order_id from " . IT_RST_REPORTING_ORDERS_DATABASE . " where status in ( " . $orderStatus  . ", 'refunded') and (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s')) group by product_type order by datetime asc $records", $startDate, $endDate, $startDate, $endDate );	
		
		$ptype = $wpdb->get_results($query);
		$ptypeA = array();
		
		foreach ( $ptype as $item ) {			
			array_push( $ptypeA, array( 'product_type' => $item->product_type, 'total' =>  strip_tags( wc_price( $item->total ) ), 'total_qtt' => $item->qtt, 'total_ref' =>   $item->qtt_ref, 'avg_price' =>  strip_tags( wc_price( $item->avgprice ) ) ) );
		}
		
		return $ptypeA;
	}
	
	/*
	 * Best Categories - Table
	*/
	public function bestCategories_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();
	
		$query = $wpdb->prepare( "select sum(total_sales) as total, category, product_type, qtt from (select sum(b.price) as total_sales, b.category, b.product_type, sum(b.quantity) as qtt from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " b where b.status in (" . $orderStatus  . ", 'refunded' ) and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') group by b.category 
UNION ALL
select -sum((select sum(total_refunded) from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " where order_id=c.order_id and product=c.meta1)) as total_refunded, c.meta3, '', ''  from " . IT_RST_REPORTING_EVENTS_DATABASE . " c  where c.order_status in (" . $orderStatus  . ", 'refunded' ) and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and c.type=2 group by c.meta3)x group by category order by total desc $records", $startDate, $endDate, $startDate, $endDate );

		$categories = $wpdb->get_results( $query );
		$categoriesA = array();
					
		foreach ( $categories as $category ) {		
			if( $category->qtt ){
				
				$cat_name 	 = ( $category->category ) ? $category->category : 'Uncategorized';		
				$total_price = ( isset( $category->qtt ) && $category->qtt ) ? ( $category->total ) / $category->qtt : 0.00;
				array_push( $categoriesA, array( 'category' => $cat_name, 'total' =>  strip_tags( wc_price( $category->total ) ), 'product_type' => $category->product_type, 'total_qtt' => $category->qtt, 'avg_price' =>  strip_tags( wc_price( $total_price ) ) ) );
			}
		}
		
		return $categoriesA;
	}
	
	/*
	 * Category Distribution by Country - Table
	*/
	public function categoryCountry_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();
		
		$query = $wpdb->prepare( "select sum(total_sales) as total, category, products, sum(coupon) as coupon, country from (select sum(b.price) as total_sales, b.category, (select value_discount from " . IT_RST_REPORTING_ORDERS_DATABASE . " where order_id=b.order_id) as coupon, sum(b.quantity) as products, b.product_type, (select country from " . IT_RST_REPORTING_ORDERS_DATABASE . " where order_id=b.order_id) as country from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " b where b.status in (" . $orderStatus  . ", 'refunded' ) and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') group by b.category, country
UNION ALL
select ifnull(-sum((select sum(total_refunded) from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " where order_id=c.order_id and product=c.meta1)),0) as total_refunded, c.meta3, 0, '', '', (select country from " . IT_RST_REPORTING_ORDERS_DATABASE . " where order_id=c.order_id) as country from " . IT_RST_REPORTING_EVENTS_DATABASE . " c  where c.order_status in (" . $orderStatus  . ", 'refunded' ) and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and c.type=2 group by c.meta3, country) x group by category, country order by total desc $records", $startDate, $endDate, $startDate, $endDate );

		$catcount  = $wpdb->get_results( $query );								
		$catcountA = array();
		
		foreach ( $catcount as $item ) {			
			$category_name = ( $item->category ) ? $item->category : 'Uncategorized';	
			array_push( $catcountA, array( 'country' => $item->country, 'total' =>  strip_tags( wc_price( $item->total ) ), 'category' => $category_name, 'num_products' => $item->products, 'coupon' =>  strip_tags( wc_price( $item->coupon ) ) ) );
		}
		
		return $catcountA;
	}
	
	/*
	 * Spend by Week - Table
	*/
	public function spendWeek_Table( $startDate, $endDate, $records = '0' ) {

		if ( !($this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select week, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select week(date) as week, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ",'refunded' ) group by week(date) union all select week(datetime), -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in ( " . $orderStatus  . ",'refunded' ) group by week(datetime)) x group by week order by total desc $records", $startDate, $endDate, $startDate, $endDate );
			
		$weeks = $wpdb->get_results( $query );
		$weeksA = array();
		
		foreach ( $weeks as $item ) {	
			$week_number = $item->week;
			$year 		 = date("Y");
			$result 	 = '';
			for ( $day=1; $day<=7; $day++ ){
				if ($day == 1)
			    	$result .= date( 'm/d/Y', strtotime( $year."W".$week_number.$day ) ) ." - ";
			    else if ( $day == 7 )
			    	$result .= date( 'm/d/Y', strtotime( $year."W".$week_number.$day ) );
			}	
			array_push( $weeksA, array( 'week' => $item->week, 'total' =>  strip_tags( wc_price( $item->total ) ), 'period' => $result, 'num_products' => $item->products, 'coupon' =>  strip_tags( wc_price( $item->coupon ) ) ) );
		}
		
		return $weeksA;
	}
	
	/*
	 * Spend by Day of Week - Table
	*/
	public function spendDay_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select weekday, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select weekday(date) as weekday, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ",'refunded' ) group by weekday(date) union all select weekday(datetime), -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in ( " . $orderStatus  . ",'refunded' ) group by weekday(datetime)) x group by weekday order by total desc $records", $startDate, $endDate, $startDate, $endDate );	
		
		$days = $wpdb->get_results( $query );			
		$daysA = array();
					
		foreach ( $days as $day ) {		
			
			switch ( $day->weekday ) {
			    case '0':
			        $dayN = __( 'Monday', 'ithemes-sales-accelerator' );
			        break;
			    case '1':
			        $dayN = __( 'Tuesday', 'ithemes-sales-accelerator' );
			        break;
				case '2':
			        $dayN = __( 'Wednesday', 'ithemes-sales-accelerator' );
			        break;
			    case '3':
			        $dayN = __( 'Thursday', 'ithemes-sales-accelerator' );
			        break;
			    case '4':
			        $dayN = __( 'Friday', 'ithemes-sales-accelerator' );
			        break;
			    case '5':
			        $dayN = __( 'Saturday', 'ithemes-sales-accelerator' );
			        break;
			    case '6':
			        $dayN = __( 'Sunday', 'ithemes-sales-accelerator' );
			        break;
			    default:
			    	$dayN = $day->day; 
			    	break;
			}
				
			array_push( $daysA, array( 'day' => $dayN, 'total' =>  strip_tags( wc_price( $day->total ) ), 'num_orders' => $day->orders, 'num_products' => $day->products, 'coupon' =>  strip_tags( wc_price( $day->coupon ) ) ) );
		}
		
		return $daysA;
	}
	
	/*
	 * Orders by order status - Table
	*/
	public function ordersStatus_Table( $startDate, $endDate ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		global $wpdb;		
		$totalType 	 	   = $this->totalType();
		$wc_order_statuses = wc_get_order_statuses();
		$orderStatus 	   = array();
		
		foreach ( $wc_order_statuses as $k => $v ) {
			$orderStatus[] = str_replace( 'wc-', '', $k );
		}
		
		$query = $wpdb->prepare( "select status, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select status, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ('" . implode('\',\'', $orderStatus) . "') group by status union all select status, -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and order_status in ('" . implode('\',\'', $orderStatus) . "') group by status) x group by status order by total desc", $startDate, $endDate, $startDate, $endDate );	
		
		$oStatus = $wpdb->get_results( $query );
		$oStatusA = array();
		
		foreach ( $oStatus as $status ) {			
			array_push( $oStatusA, array( 'status' => $status->status, 'total' =>  strip_tags( wc_price( $status->total ) ), 'num_orders' => $status->orders, 'num_products' => $status->products, 'coupon' =>  strip_tags( wc_price( $status->coupon ) ) ) );
		}
		
		return $oStatusA;
	}
	
	/*
	 * Zone Distribution - Table
	*/
	public function zoneDist_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();
		
		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		$query = $wpdb->prepare( "select state, sum(total) as total, num_orders as orders, num_products as products, coupon from (select state, sum(total_gross) as total, count(*) as num_orders, sum(num_products) as num_products, sum(total_refunded) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ") group by state  union all select state, -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and type = '2' and status in ( " . $orderStatus  . ", 'refunded' ) group by state) x group by state order by total desc $records", $startDate, $endDate,$startDate, $endDate );	
		
		$zones = $wpdb->get_results( $query );
		$zonesA = array();
		
		foreach ( $zones as $zone ) {			
			$state = ( $zone->state ) ? $zone->state : 'Unknown';
			array_push( $zonesA, array( 'state' => $state, 'total' =>  strip_tags( wc_price( $zone->total ) ), 'num_orders' => $zone->orders, 'num_products' => $zone->products, 'coupon' =>  strip_tags( wc_price( $zone->coupon ) ) ) );
		}
		
		return $zonesA;
	}
	
	/*
	 * Spend by Hour - Table
	*/
	public function spendHour_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select weekday, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select hour(date) as weekday, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ", 'refunded' ) group by hour(date) union all select hour(datetime), -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ",'refunded' ) group by hour(datetime)) x group by weekday order by total desc $records", $startDate, $endDate, $startDate, $endDate );	
		
		$hours = $wpdb->get_results( $query );
		$hoursA = array();
		
		foreach ( $hours as $hour ) {	
			array_push( $hoursA, array( 'hour' => $hour->weekday, 'total' => strip_tags( wc_price($hour->total ) ), 'num_orders' => $hour->orders, 'num_products' => $hour->products, 'coupon' =>  strip_tags( wc_price( $hour->coupon ) ) ) );
		}
		
		return $hoursA;
	}
	
	/*
	 * Total Refunds - Table
	*/
	public function totalRefunds_Table( $startDate, $endDate, $records = '0' ) {

		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}

		if ( $records == '0' ) {
			$records = ' limit 30';
		}
		else {
			$records = '';
		}

		global $wpdb;		
		$totalType 	 = $this->totalType();
		$orderStatus = $this->orderStatus();

		$query = $wpdb->prepare( "select order_id, sum(value_refunded) as value_refunded, datetime, sum(total) as total, status from (select e.order_id, meta2 as value_refunded, datetime, total_gross as total, status from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in (" . $orderStatus  . ",'refunded') order by datetime desc) x group by order_id, datetime, status $records",$startDate, $endDate );	
		$refunds = $wpdb->get_results( $query );
		$refundsA = array();
		
		foreach ( $refunds as $refund ) {	
			array_push( $refundsA, array( 'order_id' => $refund->order_id, 'value_refunded' =>  strip_tags( wc_price( $refund->value_refunded ) ), 'datetime' => $refund->datetime, 'total' =>  strip_tags( wc_price( $refund->total ) ), 'status' => $refund->status ) );
		}
		
		return $refundsA;
	}
	
	/*
	 * Average Price per Order
	*/
	public function averagePriceOrder( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$days 	 	 = $this->diffDays( $startDate,$endDate );
		$totalType   = $this->totalType();
		$orderStatus = $this->orderStatus();
		
		// What query to execute
		if ( $days == 0 ) {
			$select = "select sum($totalType)/count(*) as avg_price, date, hour(date) as hour from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ") group by hour(date) order by hour(date) asc";
		}
		else {
			$select = "select sum($totalType)/count(*) as avg_price, DATE_FORMAT(date,'%%Y-%%m-%%d') as date from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ") group by day(date), month(date), year(date) order by date asc";
		}
			
		$query 	  = $wpdb->prepare( $select, $startDate, $endDate );
		$avgPrice = $wpdb->get_results( $query );
				
		$avgPriceA = array();
		$total = 0;
		$variation = array();
		$i = 0;

		foreach ( $avgPrice as $price ) {
			if ( $days == 0 ) {
				array_push( $avgPriceA, array( 'value' => floatval( $price->avg_price ), 'date' => $price->date, 'hour' => $price->hour ) );
			}
			else {
				array_push( $avgPriceA, array( 'value' => floatval( $price->avg_price ), 'date' => $price->date ) );
			}

			$total = $total + $price->avg_price;
			
			if ( $i != 0 ) {
				array_push( $variation, ( $price->avg_price/$avgPrice[$i-1]->avg_price ) - 1 );
			}

			$i++;
		}
		
		$newData = $this->orderArray( $startDate, $endDate, $avgPriceA );

		if ( $variation ) {
			$variationAvg = ( array_sum( $variation ) / count( $variation ) ) * 100;
		}
		else {
			$variationAvg = 0;
		}
			
		$total_val = ( $avgPrice ) ? $total/count( $avgPrice ) : 0;
			
		array_multisort( $newData['dates'], SORT_ASC, $newData['allValues'] );
		return array( 'variation' => round( $variationAvg, 2 ), 'total' => strip_tags( wc_price( $total_val ) ), 'lines' =>  $newData['allValues'] );
	}
	
	/*
	 * Orders Payed vs Orders Placed
	*/
	public function ordersPvP( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$days 	   	 = $this->diffDays( $startDate,$endDate );
		$orderStatus = $this->orderStatus();
		
		// What query to execute
		$select = "select (select sum(total_gross) from " . IT_RST_REPORTING_ORDERS_DATABASE . " where status in ('pending') and (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s')) as pending, (select sum(total) from (select sum(total_gross) as total from " . IT_RST_REPORTING_ORDERS_DATABASE . " where status in (" . $orderStatus  . " ) and (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') union all select -sum(meta2) from " . IT_RST_REPORTING_EVENTS_DATABASE . " e where e.order_status in (" . $orderStatus  . " ) and type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s')) x) as payed";

		$query 	  = $wpdb->prepare( $select, $startDate, $endDate, $startDate, $endDate, $startDate, $endDate );
		$ordersPP = $wpdb->get_results( $query );
					
		$tot_pending = ( isset( $ordersPP[0]->pending ) && $ordersPP[0]->pending > 0 )  ? $ordersPP[0]->pending : 0;
		$tot_payed   = ( isset( $ordersPP[0]->payed ) 	&& $ordersPP[0]->payed > 0 )    ? $ordersPP[0]->payed   : 0;
		
		$total 	  = $tot_pending + $tot_payed;
		$pending  = round( ( $tot_pending / $total ) * 100,0 );
		$payed    = round( ( $tot_payed / $total ) * 100,0 );

		return array( 'payed_per' =>  floatval ( $payed ), 'placed_per' => floatval( $pending ), 'payed_val' =>  strip_tags( wc_price( $ordersPP[0]->payed ) ), 'placed_val' => strip_tags( wc_price( $ordersPP[0]->pending ) ) );
	}
	
	/*
	 * Top Countries
	*/
	public function topCountries( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$days 	     = $this->diffDays( $startDate,$endDate );
		$totalType   = $this->totalType();
		$orderStatus = $this->orderStatus();
					
		$query = $wpdb->prepare( "select country, sum(total) as total, num_orders as orders, num_products as products, coupon from (select country, sum(total_gross) as total, count(*) as num_orders, sum(num_products) as num_products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ", 'refunded' ) and country <> '' group by country  union all select country, -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e inner join " . IT_RST_REPORTING_ORDERS_DATABASE . " o on e.order_id = o.order_id WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and type = '2' and order_status in ( " . $orderStatus  . ", 'refunded' ) group by country and country <> '') x group by country order by total desc limit 15", $startDate, $endDate, $startDate, $endDate );	
					
		$countries = $wpdb->get_results( $query );
		$countriesA = array();
		
		foreach ( $countries as $country ) {	
			array_push( $countriesA, array( 'name' => $country->country, 'total_app' =>  strip_tags( wc_price( $country->total ) ), 'total' => floatval ( $country->total ) ) );
		}
		
		return $countriesA;
	}
	
	
	/*
	 * Spend by Week - Graph
	*/
	public function spendWeek_Graph( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$orderStatus = $this->orderStatus();
					
		$query = $wpdb->prepare( "select week, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select week(date) as week, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ",'refunded' ) group by week(date) union all select week(datetime), -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and e.order_status in ( " . $orderStatus  . ",'refunded' ) group by week(datetime)) x group by week order by total desc", $startDate, $endDate, $startDate, $endDate );
		
		$weeks = $wpdb->get_results( $query );
		$weeksA = array();
		
		foreach ( $weeks as $week ) {
			array_push( $weeksA, array( 'name' => $week->week, 'total' =>  round( floatval( $week->total ), 2 ), 'total_app' => strip_tags( wc_price( $week->total ) ) ) );
		}
		
		return $weeksA;
	}
	
	/*
	 * Spend by Hour of day - Graph
	*/
	public function spendHour_Graph( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$orderStatus = $this->orderStatus();
					
		$query = $wpdb->prepare( "select weekday, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select hour(date) as weekday, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ", 'refunded' ) group by hour(date) union all select hour(datetime), -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and e.order_status in ( " . $orderStatus  . ",'refunded' ) group by hour(datetime)) x group by weekday order by total desc", $startDate, $endDate, $startDate, $endDate );
		
		$hours = $wpdb->get_results( $query );
		$hoursA = array();
		
		foreach ( $hours as $hour ) {
			array_push( $hoursA, array( 'name' => $hour->weekday, 'total_app' =>  strip_tags( wc_price( $hour->total ) ), 'total' =>  round( floatval ( $hour->total ), 2 ) ) );
		}
		
		return $hoursA;
	}
	
	/*
	 * Spend by day of Week - Graph
	*/
	public function spendDay_Graph( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$days 	   	 = $this->diffDays( $startDate,$endDate );
		$orderStatus = $this->orderStatus();
					
		$query = $wpdb->prepare( "select weekday, sum(total) as total, sum(orders) as orders, sum(products) as products, sum(coupon) as coupon from (select weekday(date) as weekday, sum(total_gross) as total, count(*) as orders, sum(num_products) as products, sum(value_discount) as coupon from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ",'refunded' ) group by weekday(date) union all select weekday(datetime), -sum(meta2), 0, 0 ,0 from " . IT_RST_REPORTING_EVENTS_DATABASE . " e where type = 2 and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and e.order_status in ( " . $orderStatus  . ",'refunded' ) group by weekday(datetime)) x group by weekday order by total desc", $startDate, $endDate, $startDate, $endDate );
				
		$daysG  = $wpdb->get_results( $query );
		$daysGA = array();
		
		foreach ( $daysG as $day ) {
			
			switch ($day->weekday) {
			    case '0':
			        $day_name = __( 'Monday', 'ithemes-sales-accelerator' );
			        break;
			    case '1':
			        $day_name = __( 'Tuesday', 'ithemes-sales-accelerator' );
			        break;
				case '2':
			        $day_name = __( 'Wednesday', 'ithemes-sales-accelerator' );
			        break;
			    case '3':
			        $day_name = __( 'Thursday', 'ithemes-sales-accelerator' );
			        break;
			    case '4':
			        $day_name = __( 'Friday', 'ithemes-sales-accelerator' );
			        break;
			    case '5':
			        $day_name = __( 'Saturday', 'ithemes-sales-accelerator' );
			        break;
			    case '6':
			        $day_name = __( 'Sunday', 'ithemes-sales-accelerator' );
			        break;
			}
				
			array_push( $daysGA, array( 'name' => $day_name, 'total_app' => strip_tags( wc_price( $day->total ) ), 'total' =>  round( floatval ( $day->total ), 2 ) ) );
		}
		
		return $daysGA;
	}
	
	/*
	 * Returning vs New
	*/
	public function returningvsnew( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$orderStatus = $this->orderStatus();
					
		$select = "select (select (sum(total_gross) - (select ifnull(sum(meta2),0) from " . IT_RST_REPORTING_EVENTS_DATABASE . " u where u.order_id=ro.order_id and type=2)) as total from " . IT_RST_REPORTING_ORDERS_DATABASE . " ro inner join " . $wpdb->users . " u on ro.customer = u.id WHERE (DATE_FORMAT(date,'%Y-%m-%d') BETWEEN '$startDate' AND '$endDate') and status in ( 'processing','on-hold','completed') and DATE_FORMAT(date,'%Y-%m-%d') <= DATE_FORMAT(user_registered,'%Y-%m-%d')
) as new_client, (select (sum(total_gross) - (select ifnull(sum(meta2),0) from " . IT_RST_REPORTING_EVENTS_DATABASE . " u where u.order_id=ro.order_id and type=2)) as total from " . IT_RST_REPORTING_ORDERS_DATABASE . " ro inner join " . $wpdb->users . " u on ro.customer = u.id WHERE (DATE_FORMAT(date,'%Y-%m-%d') BETWEEN '$startDate' AND '$endDate') and status in ( 'processing','on-hold','completed') and DATE_FORMAT(date,'%Y-%m-%d') >= DATE_FORMAT(user_registered,'%Y-%m-%d')) as customer";
					
		$rn 		   = $wpdb->get_results( $select );
		$tot_returning = ( isset( $rn[0]->customer )   && $rn[0]->customer > 0 )   ? $rn[0]->customer : 0;
		$tot_new       = ( isset( $rn[0]->new_client ) && $rn[0]->new_client > 0 ) ? $rn[0]->new_client : 0;
		$total 	  	   = $tot_new + $tot_returning;
		$new  		   = ( $total ) ? round( ( $tot_new / $total ) * 100,0 ) : 0;
		$customer      = ( $total ) ? round( ( $tot_returning   / $total ) * 100,0 )   : 0;

		return array( 'new_per' => floatval( $new ), 'customer_per' => floatval( $customer ), 'new_val' =>  strip_tags( wc_price( $rn[0]->new_client ) ), 'customer_val' =>  strip_tags( wc_price( $rn[0]->customer ) ) );
	}
	
	/*
	 * Average Item Price per Order
	*/
	public function averagePriceProduct( $startDate, $endDate ) {
		
		global $wpdb;	
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$orderStatus = $this->orderStatus();
		$days = $this->diffDays( $startDate,$endDate );
		
		// What query to execute
		if ( $days == 0 ) {
			$select = "select sum(avg_product)/count(*) as avg_price, date, hour(date) as hour from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ") group by hour(date) order by hour(date) asc";
		}
		else {
			$select = "select sum(avg_product)/count(*) as avg_price, DATE_FORMAT(date,'%%Y-%%m-%%d') as date from " . IT_RST_REPORTING_ORDERS_DATABASE . " WHERE (DATE_FORMAT(date,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and status in ( " . $orderStatus  . ") group by day(date), month(date), year(date) order by date asc";
		}
			
		$query 	   = $wpdb->prepare( $select, $startDate, $endDate );	
		$avgPrice  = $wpdb->get_results( $query );					
		$avgPriceA = array();
		$total 	   = 0;
		$variation = array();
		$i 		   = 0;

		foreach ( $avgPrice as $price ) {
			if ( $days == 0 ) {
				array_push( $avgPriceA, array( "value" => floatval( $price->avg_price ), "date" => $price->date, "hour" => $price->hour ) );
			}
			else {
				array_push( $avgPriceA, array( "value" => floatval( $price->avg_price ), "date" => $price->date ) );
			}

			$total = $total + $price->avg_price;
			
			if ( $i != 0 ) {
				array_push( $variation, ( $price->avg_price/$avgPrice[$i-1]->avg_price ) - 1 );
			}
			$i++;
		}
		
		$newData = $this->orderArray( $startDate, $endDate, $avgPriceA );

		if ( $variation ) {
			$variationAvg = ( array_sum( $variation ) / count( $variation ) ) * 100;
		}
		else {
			$variationAvg = 0;
		}
			
		$total_val = ( $avgPrice ) ? $total/count( $avgPrice ) : 0;
			
		array_multisort( $newData['dates'], SORT_ASC, $newData['allValues'] );
		return array( 'variation' => round( $variationAvg, 2 ), 'total' =>  strip_tags( wc_price( $total_val ) ), 'lines' =>  $newData['allValues'] );
	}
	
	/*
	 * Best Product
	*/
	public function bestProduct( $startDate, $endDate ) {
		
		global $wpdb;	
		$totalType = $this->totalType();
		
		if ( !( $this->validateDate( $startDate ) && $this->validateDate( $endDate ) ) ) {
			return array();
		}
			
		$days 		 = $this->diffDays( $startDate,$endDate );
		$orderStatus = $this->orderStatus();
					
		$best_product = "select (total - ifnull(refunded,0)) as total, product from (select sum(price) as total, product, (select sum(meta2) as total from " . IT_RST_REPORTING_EVENTS_DATABASE . " d where d.meta1=a.product and d.type=2 and d.order_status in (" . $orderStatus  . ", 'refunded') and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') group by d.meta1 ) as refunded from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " a where status in (" . $orderStatus  . ", 'refunded') and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') group by product, day(datetime), month(datetime), year(datetime) order by total desc) x order by total desc limit 1";

		$query = $wpdb->prepare( $best_product, $startDate, $endDate, $startDate, $endDate );
		$bestProduct = $wpdb->get_row( $query );
		
		if( !empty( $bestProduct ) ) {
		
			// What query to execute
			if ( $days == 0 ) {
				$select = "select product, date, sum(total) as total, hour(date) as hour from (select x.product, x.date, sum(total) as total from (select product, datetime as date, sum(price) as total from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and product = " . $bestProduct->product . " and status in (" . $orderStatus  . ", 'refunded') group by hour(datetime) union all select meta1, datetime, -sum(meta2) from " . IT_RST_REPORTING_EVENTS_DATABASE . " where type = 2 and meta1 = " . $bestProduct->product . " and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s')  group by hour(datetime)) x group by product, hour(date)) x group by product, hour(date)";
			}
			else {
				$select = "select product, date, sum(total) as total from (select x.product, x.date, sum(total) as total from (select product, datetime as date, sum(price) as total from " . IT_RST_REPORTING_ORDER_PRODUCTS_DATABASE . " WHERE (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and product = " . $bestProduct->product . " and status in (" . $orderStatus  . ", 'refunded') group by day(datetime), month(datetime) union all select meta1, datetime, -sum(meta2) from " . IT_RST_REPORTING_EVENTS_DATABASE . " where type = 2 and meta1 = " . $bestProduct->product . " and (DATE_FORMAT(datetime,'%%Y-%%m-%%d') BETWEEN '%s' AND '%s') and meta1 <> 0 and order_status in (" . $orderStatus  . ", 'refunded') group by day(datetime), month(datetime), year(datetime)) x group by product, day(date), month(date), year(date)) x group by product, day(date), month(date), year(date), year(date)";
			}
			
			$query = $wpdb->prepare( $select, $startDate, $endDate, $startDate, $endDate );	
			$bestP = $wpdb->get_results( $query );
			
		}
		else{
			$bestP = array();
		}
		
		$bestPA    = array();
		$total 	   = 0;
		$variation = array();
		$i 		   = 0;

		foreach ( $bestP as $product ) {
			if ( $days == 0 ) {
				array_push( $bestPA, array( "value" => floatval( $product->total ), "date" => $product->date, "hour" => $product->hour ) );
			}
			else {
				array_push( $bestPA, array( "value" => floatval( $product->total ), "date" => $product->date ) );
			}

			$total = $total + $product->total;
			
			if ( $i != 0 ) {
				array_push( $variation, ( $product->total/$bestP[$i-1]->total ) - 1 );
			}

			$i++;
		}
		
		$newData = $this->orderArray( $startDate, $endDate, $bestPA );
		
		if ( $variation ) {
			$variationAvg = ( array_sum( $variation ) / count( $variation ) ) * 100;
		}
		else {
			$variationAvg = 0;
		}
			
		$name = isset( $bestP[0]->product ) ? get_the_title( $bestP[0]->product ) : '';
			
		array_multisort( $newData['dates'], SORT_ASC, $newData['allValues'] );
		return array( 'name' => $name, 'variation' => round( $variationAvg, 2 ), 'total' =>  strip_tags( wc_price( $total ) ), 'lines' =>  $newData['allValues'] );
	}
}