<?php

if ( !defined( 'ABSPATH' ) ) {
	exit(); // Exit if accessed directly
}

/**
* This class handles module Settings
**/

class RST_Reporting_Settings_Pro {
	
	// Singleton design pattern
    protected static $instance = NULL;
    
    // Method to return the singleton instance
    public static function get_instance() {
	    
        if ( null == self::$instance ) {
            self::$instance = new self();
        }
        
        return self::$instance;
    }
	
	public function __construct() {
		   
		// Add settings to created tab
		add_filter( 'it_rst_filter_settings', array( $this, 'add_dashboard_settings_pro' ), 50, 1 );	
		
		// Adds Report Options to Reporting Settings tab
		add_action( 'it_rst_reporting_dashboard_settings_grid_help', array( $this, 'add_grid_help_option' ) );
		
		// Report list timer shortcode
		add_shortcode( 'rst_reports_list_timer', array( $this, 'rst_reportsListTimer' ) );		
	}	
	
	public function add_dashboard_settings_pro( $settings ) {
		
		foreach ( $settings as $k => $setting ) {
			if ( $setting['name'] == 'reporting_settings' ) {
				
				$settings[ $k ]['settings'][] = array(
					'name'  => 'it_rooster_reporting_product_cost',
					'class' => $this,
					'title' => __( 'Product Cost<br><span class="desc">Activate a new field to define the cost of a product and include in Report dashboard with Product Cost vs Product Sale.</span>', 'ithemes-sales-accelerator' ),
				);
				
				$settings[ $k ]['settings'][] = array(
					'name' 	=> 'it_rooster_reporting_live_update',
					'class' => $this,
					'title' => __( 'Live Update<br><span class="desc">Activate live updates of the Reports dashboard data.</span>', 'ithemes-sales-accelerator' )
				);
				
				break;
			}
		}
		
		return $settings;
	}
	
	// Set live update (on/off)
	public function it_rooster_reporting_live_update() {
		
		global $it_rst_reporting_settings;
		$settings = $it_rst_reporting_settings;
        $value 	  = isset( $settings['live_update'] ) ? $settings['live_update'] : '';
        $checked  = ( $value ) ? 'checked ' : '';
        
        echo '<label class="switch-modules">
			        <input name="it_rooster_reporting_settings[live_update]" id="it_rooster_reporting_settings[live_update]" value="0" type="hidden">
					<input ' . $checked . 'class="it_rooster_checkbox" name="it_rooster_reporting_settings[live_update]" id="it_rooster_reporting_settings[live_update]" value="1" type="checkbox">
					<span class="slider-m round-m"></span>
			  </label>';
	}
	
	// Set product cost (on/off)
	public function it_rooster_reporting_product_cost() {
		
		global $it_rst_reporting_settings;
		$settings = $it_rst_reporting_settings;
        $value 	  = isset( $settings['product_cost'] ) ? $settings['product_cost'] : '';
        $checked  = ( $value ) ? 'checked ' : '';
        
        echo '<label class="switch-modules">
			        <input name="it_rooster_reporting_settings[product_cost]" id="it_rooster_reporting_settings[product_cost]" value="0" type="hidden">
					<input ' . $checked . 'class="it_rooster_checkbox" name="it_rooster_reporting_settings[product_cost]" id="it_rooster_reporting_settings[product_cost]" value="1" type="checkbox">
					<span class="slider-m round-m"></span>
			  </label>';
	}
	
	public function add_grid_help_option(){
		echo '<div class="openModal_Timer"><span class="last_btts" style="display:inline-flex; text-align: center;">' . __( 'Live Update', 'ithemes-sales-accelerator' ) . '<br>' . __( 'Options', 'ithemes-sales-accelerator' ) . '</span>

				<! --- Modal --- !>
				<div class="modal-modules">
					<div id="modal_content">
						<div class="inline" style="width:100%; position: relative; margin-bottom:30px;">
							<div class="name_desc" style="margin:0;">
								<h5>' . __( 'List of all Reports', 'ithemes-sales-accelerator' ) . '</h5>
							</div>
							<div style="position: absolute; right: 0;">
								<svg class="closeModal" width="17px" height="17px" viewBox="0 0 17 17" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
								    <defs></defs>
								    <g id="Modules" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
								        <g id="Modules---Welcome" transform="translate(-993.000000, -704.000000)" fill-rule="nonzero" fill="#D4D4D4">
								            <g id="Modal-Info" transform="translate(418.000000, 664.000000)">
								                <g id="Group-2" transform="translate(575.000000, 40.000000)">
								                    <path d="M8.5,7.08578644 L1.70710678,0.292893219 C1.31658249,-0.0976310729 0.683417511,-0.0976310729 0.292893219,0.292893219 C-0.0976310729,0.683417511 -0.0976310729,1.31658249 0.292893219,1.70710678 L7.08578644,8.5 L0.55714681,15.0286396 C0.166622518,15.4191639 0.166622518,16.0523289 0.55714681,16.4428532 C0.947671102,16.8333775 1.58083608,16.8333775 1.97136037,16.4428532 L8.5,9.91421356 L15.0286396,16.4428532 C15.4191639,16.8333775 16.0523289,16.8333775 16.4428532,16.4428532 C16.8333775,16.0523289 16.8333775,15.4191639 16.4428532,15.0286396 L9.91421356,8.5 L16.7071068,1.70710678 C17.0976311,1.31658249 17.0976311,0.683417511 16.7071068,0.292893219 C16.3165825,-0.0976310729 15.6834175,-0.0976310729 15.2928932,0.292893219 L8.5,7.08578644 L8.5,7.08578644 Z" id="Combined-Shape"></path>
								                </g>
								            </g>
								        </g>
								    </g>
								</svg>
							</div>
							
						</div>
						<div class="content_modal">
							' . do_shortcode( '[rst_reports_list_timer]' ) . '
							<div style="margin-top:30px; width:100%; text-align:center;"><span id="it_rooster_reports_list_timer_btn" class="btt_upgrade">' . __( 'Save Changes', 'ithemes-sales-accelerator' ) . '</span></div>
						</div>
						
					</div>
				</div>
				<! --- Modal --- !>
			</div>
		  </div>';
	}
	
	// Shortcode to Reports List 
	public function rst_reportsListTimer() {
		
		$features 	  = RST_Reporting_Features::get_instance();
		$feature_list = $features->get_features();
		
		$html = '<form id="it_rooster_reports_list_timer">
					<div class="list">
						<div class="row npm">
								<div class="col s6">Title</div>
								<div class="col s3">Time to refresh (ms)</div>
								<div class="col s3">Live Update?</div>
						</div>';
						
						foreach ( $feature_list as $feature ){
							$value 		= $feature['value'];
							$key 		= $feature['key'];
							$refresh    = $feature['refresh'];
							$live       = ( $feature['live'] ) ? "#4DC2F0" : "#FFFFFF";
							$checked    = ( $feature['live'] ) ? " checked" : "";
							$show       = ( ( $feature['premium'] && defined( 'IT_RST_PLUGIN_PREMIUM_ACTIVE' ) || !$feature['premium'] ) ) ? true : false;
							
							// Show only premium features to premium members
							if ( $show ){					
								$html .= '<div class="row npm">
									<div class="col s6">' . $value . '</div>
									<div class="col s3"><input name="it_rooster_reporting_features[' . $key . '][refresh]" id="it_rooster_reporting_features[' . $key . '][refresh]" type="number" min="5000" max="600000" value="' . $refresh . '" step="100"></div>
									<div class="col s3">
									
										<input name="it_rooster_reporting_features[' . $key . '][live]" id="it_rooster_reporting_features[' . $key . '][live]" value="0" type="hidden">
										<input class="it_rooster_checkbox" id="it_rooster_reporting_features_' . $key . '_live" name="it_rooster_reporting_features[' . $key . '][live]" id="it_rooster_reporting_features[' . $key . '][live]" value="1" type="checkbox" ' . $checked . '>
										<svg width="20px" height="20px" data-element-key="' . $key . '" class="it_rst_svg_features_circle" viewBox="0 0 20 20" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">
										    <!-- Generator: Sketch 45.2 (43514) - http://www.bohemiancoding.com/sketch -->
										    <desc>Created with Sketch.</desc>
										    <defs>
										        <rect id="path-1" x="94" y="687" width="20" height="20" rx="10"></rect>
										    </defs>
										    <g id="Components" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
										        <g id="Guide-02-Avenir-Copy-2" transform="translate(-94.000000, -687.000000)">
										            <g id="Rectangle-Copy-6">
										                <use fill="' . $live . '" fill-rule="evenodd" xlink:href="#path-1" id="circle_' . $key . '_live' . '"></use>
										                <rect stroke="#DFDFDF" stroke-width="1" x="94.5" y="687.5" width="19" height="19" rx="9.5"></rect>
										            </g>
										            <circle id="Oval" fill="#FFFFFF" cx="104" cy="697" r="3"></circle>
										        </g>
										    </g>
										</svg> 
									</div>
						</div>';
						}
					}
		$html .= '</form></div>';		
		return $html;
	}
}
